import os
import sys
import yaml
import json
import pickle
from .base import Base
from .utils import dir_metatree

"""The init command."""


class Init(Base):

    def run(self):

        data = dir_metatree(
            _ROOT_=self.options.get('<path>') or '.',
            ignore_dot_files=not self.options.get('--all')
        )

        as_string = yaml.dump(data, allow_unicode=True)

        content = '# FORMAT\n```yaml\n{}```'.format(as_string)

        print(content)



import os
import re
import fnmatch
import copy


def nested_set(dic, keys, value):
    for key in keys[:-1]:
        dic = dic.setdefault(key, {'*': ''})
    dic[keys[-1]] = value


def dir_metatree(
        _ROOT_='.',
        ignore_dot_files=False,
        listen_gitignore=True):

    _LEAF_ = {'*': ''}

    data = copy.copy(_LEAF_)

    for dirName, subdirList, fileList in os.walk(_ROOT_):
        dirPath = dirName.split('/')[1:]

        if ignore_dot_files:

            if any([name.startswith('.') for name in dirPath]):
                continue

        for fname in fileList:

            if ignore_dot_files:
                if fname.startswith('.'):
                    continue

            if listen_gitignore:
                if os.path.exists('.gitignore'):
                    pass

                    path = os.path.join(dirName, fname)

                    patterns = [line[:-1] for line in
                               open('.gitignore').readlines()
                                if not line.startswith('#') and line[:-1]]

                    any_matched = False

                    for pattern in patterns:
                        result = re.search(fnmatch.translate(pattern), path)
                        if result:
                            any_matched = True

                    if any_matched:
                        continue



            if dirPath:
                nested_set(data, dirPath+[fname], copy.copy(_LEAF_))
            else:
                data[fname] = copy.copy(_LEAF_)

    return data





"""Tests for our `metadir .` subcommand."""


from subprocess import PIPE, Popen as popen
from unittest import TestCase


class TestHello(TestCase):
    def test_returns_multiple_lines(self):
        output = popen(['metadir', '.'], stdout=PIPE).communicate()[0]
        lines = str(output, 'utf-8').split('\n')
        self.assertTrue(len(lines) > 0)



"""
metadir

Usage:
  metadir <path>
  metadir <path> -a | --all
  metadir -h | --help
  metadir --version

Options:
  -a --all                          Does not ignore dot files
  -i --ignore                       Regex to igore
  -h --help                         Show this screen.
  --version                         Show version.

Examples:
  metadir

Help:
  For help using this tool, please open an issue on the Github repository:
  https://gitlab.com/wefindx/metadir
"""


from inspect import getmembers, isclass

from docopt import docopt

from . import __version__ as VERSION


def main():
    """Main CLI entrypoint."""
    import metadir.commands
    options = docopt(__doc__, version=VERSION)
    # print(options)

    for (k, v) in options.items():

        if options.get('<path>'):
            k = 'init'

        if hasattr(metadir.commands, k) and v:
            module = getattr(metadir.commands, k)
            metadir.commands = getmembers(module, isclass)
            command = [command[1] for command in metadir.commands if command[0] != 'Base'][0]
            command = command(options)
            command.run()
